@@ -46,31 +46,15 @@ for sourceview in "gtksourceview sourceview".split():
 
 if sourceview_available:
     def set_highlighting_enabled(buf, fname, enabled):
-        # gnome.vfs.get_mime_type seems to be broken. fake it.
-        extmap = { "xml":"text/xml",
-                   "glade":"text/xml",
-                   "cpp":"text/x-cpp",
-                   "cxx":"text/x-cpp",
-                   "cc":"text/x-cpp",
-                   "C":"text/x-cpp",
-                   "c":"text/x-c",
-                   "hpp":"text/x-cpp",
-                   "hxx":"text/x-cpp",
-                   "hh":"text/x-cpp",
-                   "H":"text/x-cpp",
-                   "h":"text/x-cpp",
-                   "inl":"text/x-cpp",
-                   "desktop": "application/x-desktop",
-                   "diff": "text/x-diff",
-                   "patch": "text/x-diff",
-                   "html": "text/html",
-                   "po": "text/x-po",
-                   "py": "text/x-python" }
-        ext = fname.split(".")[-1]
-        man = gsv.SourceLanguagesManager()
-        gsl = man.get_language_from_mime_type( extmap.get(ext, "text/plain") )
-        if gsl:
-            buf.set_language(gsl)
+        if enabled:
+            import gnomevfs
+            mime_type = gnomevfs.get_mime_type( os.path.abspath(fname) )
+            man = gsv.SourceLanguagesManager()
+            gsl = man.get_language_from_mime_type( mime_type )
+            if gsl:
+                buf.set_language(gsl)
+            else:
+                enabled = False
         buf.set_highlight(enabled)
 
 gdk = gtk.gdk
@@ -205,8 +189,9 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             id1 = buf.connect("delete-range", self.on_text_delete_range)
             id2 = buf.connect_after("insert-text", self.after_text_insert_text)
             id3 = buf.connect_after("delete-range", self.after_text_delete_range)
+            id4 = buf.connect("mark-set", self.on_textbuffer_mark_set)
             buf.textview = textview
-            buf.handlers = id0, id1, id2, id3
+            buf.handlers = id0, id1, id2, id3, id4
 
     def _update_cursor_status(self, buf):
         def update():
@@ -362,8 +347,8 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         state = [b.modified for b in self.bufferdata]
         return 1 in state
 
-    def _get_filename(self, i):
-        return self.bufferdata[i].filename or "<unnamed>"
+    def _get_pane_label(self, i):
+        return self.bufferdata[i].label or "<unnamed>"
 
     def on_delete_event(self, appquit=0):
         modified = [b.modified for b in self.bufferdata]
@@ -372,7 +357,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             dialog.widget.set_transient_for(self.widget.get_toplevel())
             buttons = []
             for i in range(self.num_panes):
-                b = gtk.CheckButton( self._get_filename(i) )
+                b = gtk.CheckButton( self._get_pane_label(i) )
                 b.set_use_underline(False)
                 buttons.append(b)
                 dialog.box.pack_start(b, 1, 1)
@@ -380,7 +365,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     b.set_sensitive(0)
                 else:
                     b.set_active(1)
-            dialog.widget.show_all()
+            dialog.box.show_all()
             if not appquit:
                 dialog.button_quit.hide()
             response = dialog.widget.run()
@@ -540,10 +525,15 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         # text buffer loading/saving
         #
 
+    def set_labels(self, lst):
+        assert len(lst) <= len(self.bufferdata)
+        for l,d in zip(lst,self.bufferdata):
+            if len(l): d.label = l
+
     def recompute_label(self):
         filenames = []
         for i in range(self.num_panes):
-            filenames.append( self._get_filename(i) )
+            filenames.append( self._get_pane_label(i) )
         shortnames = misc.shorten_names(*filenames)
         for i in range(self.num_panes):
             stock = None
@@ -573,7 +563,10 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 b.delete( b.get_start_iter(), b.get_end_iter() )
                 absfile = os.path.abspath(f)
                 self.fileentry[i].set_filename(absfile)
-                self.bufferdata[i] = MeldBufferData(absfile)
+                bold, bnew = self.bufferdata[i], MeldBufferData(absfile)
+                if bold.filename == bnew.filename:
+                    bnew.label = bold.label
+                self.bufferdata[i] = bnew
         self.recompute_label()
         self.scheduler.add_task( self._set_files_internal(files).next )
 
@@ -758,6 +751,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
             parent=self.widget.get_toplevel(),
             action=gtk.FILE_CHOOSER_ACTION_SAVE,
             buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK) )
+        dialog.set_default_response(gtk.RESPONSE_OK)
         response = dialog.run()
         filename = None
         if response == gtk.RESPONSE_OK:
@@ -790,7 +784,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if saveas or not bufdata.filename:
             filename = self._get_filename_for_saving( _("Choose a name for buffer %i.") % (pane+1) )
             if filename:
-                bufdata.filename = os.path.abspath(filename)
+                bufdata.filename = bufdata.label = os.path.abspath(filename)
                 self.fileentry[pane].set_filename( bufdata.filename)
             else:
                 return melddoc.RESULT_ERROR
@@ -801,7 +795,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                     text = text.replace("\n", bufdata.newlines)
             elif type(bufdata.newlines) == type(()):
                 buttons = {'\n':("UNIX (LF)",0), '\r\n':("DOS (CR-LF)", 1), '\r':("MAC (CR)",2) }
-                newline = misc.run_dialog( _("This file '%s' contains a mixture of line endings.\n\nWhich format would you like to use?") % bufdata.filename,
+                newline = misc.run_dialog( _("This file '%s' contains a mixture of line endings.\n\nWhich format would you like to use?") % bufdata.label,
                     self, gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_CANCEL,
                     extrabuttons=[ buttons[b] for b in bufdata.newlines ] )
                 if newline < 0:
@@ -817,7 +811,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
                 text = text.encode(bufdata.encoding)
             except UnicodeEncodeError:
                 if misc.run_dialog(
-                    _("'%s' contains characters not encodable with '%s'\nWould you like to save as UTF-8?") % (bufdata.filename, bufdata.encoding),
+                    _("'%s' contains characters not encodable with '%s'\nWould you like to save as UTF-8?") % (bufdata.label, bufdata.encoding),
                     self, gtk.MESSAGE_ERROR, gtk.BUTTONS_YES_NO) != gtk.RESPONSE_YES:
                     return melddoc.RESULT_ERROR
         if self._save_text_to_filename(bufdata.filename, text):
@@ -830,23 +824,39 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
 
     def make_patch(self, pane):
         fontdesc = pango.FontDescription(self.prefs.get_current_font())
-        dialog = gnomeglade.Component( paths.share_dir("glade2/filediff.glade"), "patchdialog")
+        override = {}
+        if sourceview_available:
+            override["GtkTextView"] = gsv.SourceView
+            override["GtkTextBuffer"] = gsv.SourceBuffer
+        dialog = gnomeglade.Component( paths.share_dir("glade2/filediff.glade"), "patchdialog", override)
         dialog.widget.set_transient_for( self.widget.get_toplevel() )
         bufs = [t.get_buffer() for t in self.textview]
         texts = [b.get_text(*b.get_bounds()).split("\n") for b in bufs]
         texts[0] = [l+"\n" for l in texts[0]]
         texts[1] = [l+"\n" for l in texts[1]]
-        names = [self._get_filename(i) for i in range(2)]
+        names = [self._get_pane_label(i) for i in range(2)]
+        prefix = os.path.commonprefix( names )
+        try: prefixslash = prefix.rindex("/") + 1
+        except ValueError: prefixslash = 0
+        names = [n[prefixslash:] for n in names]
+        if sourceview_available:
+            dialog.textview.set_buffer( gsv.SourceBuffer() )
         dialog.textview.modify_font(fontdesc)
         buf = dialog.textview.get_buffer()
         lines = []
         for line in difflib.unified_diff(texts[0], texts[1], names[0], names[1]):
             buf.insert( buf.get_end_iter(), line )
             lines.append(line)
+        if sourceview_available:
+            man = gsv.SourceLanguagesManager()
+            gsl = man.get_language_from_mime_type("text/x-diff")
+            if gsl:
+                buf.set_language(gsl)
+                buf.set_highlight(True)
         result = dialog.widget.run()
         dialog.widget.destroy()
         if result >= 0:
-            txt = "".join(lines)
+            txt = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), 0)
             if result == 1: # copy
                 clip = gtk.clipboard_get()
                 clip.set_text(txt)
@@ -904,7 +914,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         # refresh and reload
         #
     def on_reload_activate(self, *extra):
-        modified = [b.filename for b in self.bufferdata if b.modified]
+        modified = [os.path.basename(b.label) for b in self.bufferdata if b.modified]
         if len(modified):
             message = _("Reloading will discard changes in:\n%s\n\nYou cannot undo this operation.") % "\n".join(modified)
             response = misc.run_dialog( message, parent=self, messagetype=gtk.MESSAGE_WARNING, buttonstype=gtk.BUTTONS_OK_CANCEL)
@@ -942,7 +952,7 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         # only allow one scrollbar to be here at a time
         if not hasattr(self,"_sync_vscroll_lock"):
             self._sync_vscroll_lock = 0
-        if not self._sync_vscroll_lock:
+        if (self.keymask & MASK_SHIFT)==0 and not self._sync_vscroll_lock:
             self._sync_vscroll_lock = 1
             syncpoint = 0.5
 
@@ -1020,9 +1030,14 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         gc = area.meldgc.get_gc
         for c in self.linediffer.single_changes(textindex, self._get_texts()):
             assert c[0] != "equal"
+            outline = True
+            if self.prefs.ignore_blank_lines:
+                c1,c2 = self._consume_blank_lines( self._get_texts()[textindex][c[1]:c[2]] )
+                if (c1 or c2) and (c[1]+c1 == c[2]-c2):
+                    outline = False
             s,e = [int(x) for x in ( math.floor(scaleit(c[1])), math.ceil(scaleit(c[2]+(c[1]==c[2]))) ) ]
             window.draw_rectangle( gc(c[0]), 1, x0, s, x1, e-s)
-            window.draw_rectangle( gctext, 0, x0, s, x1, e-s)
+            if outline: window.draw_rectangle( gctext, 0, x0, s, x1, e-s)
 
     def on_diffmap_button_press_event(self, area, event):
         #TODO need gutter of scrollbar - how do we get that?
@@ -1074,10 +1089,16 @@ class FileDiff(melddoc.MeldDoc, gnomeglade.Component):
         if direction == gdk.SCROLL_DOWN:
             for c in self.linediffer.single_changes(1, self._get_texts()):
                 assert c[0] != "equal"
+                c1,c2 = self._consume_blank_lines( self._get_texts()[1][c[1]:c[2]] )
+                if c[1]+c1 == c[2]-c2:
+                    continue
                 if c[1] > curline + 1:
                     break
         else: #direction == gdk.SCROLL_UP
             for chunk in self.linediffer.single_changes(1, self._get_texts()):
+                c1,c2 = self._consume_blank_lines( self._get_texts()[1][chunk[1]:chunk[2]] )
+                if chunk[1]+c1 == chunk[2]-c2:
+                    continue
                 if chunk[2] < curline:
                     c = chunk
                 elif c:
@@ -1343,11 +1364,12 @@ if gobject.pygtk_version < (2,8,0):
 ################################################################################
 
 class MeldBufferData(object):
-    __slots__ = ("modified", "writable", "filename", "encoding", "newlines")
+    __slots__ = ("modified", "writable", "filename", "label", "encoding", "newlines")
     def __init__(self, filename=None):
         self.modified = 0
         self.writable = 1
         self.filename = filename
+        self.label = filename
         self.encoding = None
         self.newlines = None
 
