@@ -1,4 +1,4 @@
-### Copyright (C) 2002-2005 Stephen Kennedy <stevek@gnome.org>
+### Copyright (C) 2002-2006 Stephen Kennedy <stevek@gnome.org>
 
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ for sourceview in "gtksourceview sourceview".split():
     except ImportError:
         pass
 
-version = "1.1.3"
+version = "1.1.5.1"
 
 # magic developer switch, changes some behaviour
 developer = 0
@@ -89,6 +89,8 @@ class NewDocDialog(gnomeglade.Component):
             paths = [ e.get_full_path(0) or "" for e in self.entrylists[page] ]
             if page < 2 and not self.three_way_compare[page].get_active():
                 paths.pop(0)
+            for path in paths:
+                self.entrylists[page][0].gnome_entry().append_history(True, path)
             methods = (self.parentapp.append_filediff,
                        self.parentapp.append_dirdiff,
                        self.parentapp.append_vcview )
@@ -142,8 +144,8 @@ class ListWidget(gnomeglade.Component):
     def on_item_new_clicked(self, button):
         model = self.treeview.get_model()
         it = model.append()
-        model.set_value(it, 0, "label")
-        model.set_value(it, 2, "pattern")
+        model.set_value(it, 0, _("label"))
+        model.set_value(it, 2, _("pattern"))
         self._update_filter_string()
     def _get_selected(self):
         selected = []
@@ -244,11 +246,12 @@ class PreferencesDialog(gnomeglade.Component):
         self.draw_style[self.prefs.draw_style].set_active(1)
         self.toolbar_style[self.prefs.toolbar_style].set_active(1)
         # file filters
-        cols = [ ("Name", type("")), ("Active", type(0)), ("Pattern", type("")) ]
+        cols = [ (_("Name"), type("")), (_("Active"), type(0)), (_("Pattern"), type("")) ]
         self.filefilter = ListWidget( cols, self.prefs, "filters")
         self.file_filters_box.pack_start(self.filefilter.widget)
+        self.checkbutton_ignore_symlinks.set_active( self.prefs.ignore_symlinks)
         # text filters
-        cols = [ ("Name", type("")), ("Active", type(0)), ("Regex", type("")) ]
+        cols = [ (_("Name"), type("")), (_("Active"), type(0)), (_("Regex"), type("")) ]
         self.textfilter = ListWidget( cols, self.prefs, "regexes")
         self.text_filters_box.pack_start(self.textfilter.widget)
         self.checkbutton_ignore_blank_lines.set_active( self.prefs.ignore_blank_lines )
@@ -289,11 +292,11 @@ class PreferencesDialog(gnomeglade.Component):
     def on_checkbutton_show_line_numbers_toggled(self, check):
         self.prefs.show_line_numbers = check.get_active()
         if check.get_active() and not sourceview_available:
-            misc.run_dialog(_("Line numbers are only available if you have pysourceview installed.") )
+            misc.run_dialog(_("Line numbers are only available if you have gnome-python-desktop installed.") )
     def on_checkbutton_use_syntax_highlighting_toggled(self, check):
         self.prefs.use_syntax_highlighting = check.get_active()
         if check.get_active() and not sourceview_available:
-            misc.run_dialog(_("Syntax highlighting is only available if you have pysourceview installed.") )
+            misc.run_dialog(_("Syntax highlighting is only available if you have gnome-python-desktop installed.") )
     def on_editor_command_toggled(self, radio):
         if radio.get_active():
             idx = self.editor_command.index(radio)
@@ -313,6 +316,8 @@ class PreferencesDialog(gnomeglade.Component):
     #
     # filters
     #
+    def on_checkbutton_ignore_symlinks_toggled(self, check):
+        self.prefs.ignore_symlinks = check.get_active()
     def on_checkbutton_ignore_blank_lines_toggled(self, check):
         self.prefs.ignore_blank_lines = check.get_active()
 
@@ -377,13 +382,14 @@ class NotebookLabel(gtk.HBox):
         gtk.HBox.__init__(self)
         self.label = gtk.Label(text)
         self.button = gtk.Button()
+        self.button.set_relief(gtk.RELIEF_NONE)
+        image = gtk.Image()
+        image.set_from_stock( gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
+        image.set_size_request( 10,10 ) #TODO font height
+        self.button.add( image )
         icon = gtk.Image()
         icon.set_from_file( paths.share_dir("glade2/pixmaps/%s" % iconname) )
-        icon.set_from_pixbuf( icon.get_pixbuf().scale_simple(15, 15, 2) ) #TODO font height
-        image = gtk.Image()
-        image.set_from_file( paths.share_dir("glade2/pixmaps/button_delete.xpm") )
-        image.set_from_pixbuf( image.get_pixbuf().scale_simple(9, 9, 2) ) #TODO font height
-        self.button.add( image )
+        icon.set_from_pixbuf( icon.get_pixbuf().scale_simple(15, 15, 2) ) #TODO stock image
         self.pack_start( icon )
         self.pack_start( self.label )
         self.pack_start( self.button, expand=0 )
@@ -413,6 +419,7 @@ class MeldPreferences(prefs.Preferences):
         "save_encoding": prefs.Value(prefs.INT, 0),
         "draw_style": prefs.Value(prefs.INT,2),
         "toolbar_style": prefs.Value(prefs.INT,0),
+        "ignore_symlinks": prefs.Value(prefs.BOOL,0),
         "cvs_quiet": prefs.Value(prefs.BOOL, 1),
         "cvs_compression": prefs.Value(prefs.BOOL, 1),
         "cvs_compression_value": prefs.Value(prefs.INT, 3),
@@ -431,18 +438,27 @@ class MeldPreferences(prefs.Preferences):
         "color_inline_fg" : prefs.Value(prefs.STRING, "Red"),
         "color_edited_bg" : prefs.Value(prefs.STRING, "gray90"),
         "color_edited_fg" : prefs.Value(prefs.STRING, "Black"),
-        "filters" : prefs.Value(prefs.STRING, "Backups\t1\t#*# .#* ~* *~ *.{orig,bak,swp}\n" + \
-                                              "CVS\t1\tCVS\n" + \
-                                              "SVN\t1\t.svn\n" + \
-                                              "Monotone\t1\tMT\n" + \
-                                              "Binaries\t1\t*.{pyc,a,obj,o,so,la,lib,dll}\n" + \
-                                              "Media\t0\t*.{jpg,gif,png,wav,mp3,ogg,xcf,xpm}"),
-        "regexes" : prefs.Value(prefs.STRING, "CVS keywords\t0\t\$\\w+(:[^\\n$]+)?\$\n" + \
-                                              "C++ comment\t0\t//.*\n" + \
-                                              "C comment\t0\t/\*.*?\*/\n" + \
-                                              "All whitespace\t0\t[ \\t\\r\\f\\v]*\n" + \
-                                              "Leading whitespace\t0\t^[ \\t\\r\\f\\v]*\n" + \
-                                              "Script comment\t0\t#.*"),
+        "filters" : prefs.Value(prefs.STRING,
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("Backups\t1\t#*# .#* ~* *~ *.{orig,bak,swp}\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("Version Control\t1\tCVS .svn MT [{]arch[}] .arch-ids .arch-inventory\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("Binaries\t1\t*.{pyc,a,obj,o,so,la,lib,dll}\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("Media\t0\t*.{jpg,gif,png,wav,mp3,ogg,xcf,xpm}")),
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            "regexes" : prefs.Value(prefs.STRING, _("CVS keywords\t0\t\$\\w+(:[^\\n$]+)?\$\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("C++ comment\t0\t//.*\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("C comment\t0\t/\*.*?\*/\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("All whitespace\t0\t[ \\t\\r\\f\\v]*\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("Leading whitespace\t0\t^[ \\t\\r\\f\\v]*\n") + \
+            #TRANSLATORS: translate this string ONLY to the first "\t", leave it and the following parts intact
+            _("Script comment\t0\t#.*")),
         "ignore_blank_lines" : prefs.Value(prefs.BOOL, 1)
     }
 
@@ -544,10 +560,12 @@ class MeldApp(gnomeglade.GnomeApp):
         else:
             self.statusbar.task_progress.set_fraction(0)
         if self.scheduler.tasks_pending():
+            self.toolbar_stop.set_sensitive(1)
             return 1
         else:
             self.statusbar.set_task_status("")
             self.idle_hooked = 0
+            self.toolbar_stop.set_sensitive(0)
             return 0
 
     def on_scheduler_runnable(self, sched):
@@ -636,6 +654,12 @@ class MeldApp(gnomeglade.GnomeApp):
 
     def on_menu_redo_activate(self, *extra):
         self.current_doc().on_redo_activate()
+
+    def on_menu_refresh_activate(self, *extra):
+        self.current_doc().on_refresh_activate()
+
+    def on_menu_reload_activate(self, *extra):
+        self.current_doc().on_reload_activate()
   
     def on_menu_find_activate(self, *extra):
         self.current_doc().on_find_activate()
@@ -710,6 +734,8 @@ class MeldApp(gnomeglade.GnomeApp):
             i = self.notebook.page_num( page.widget )
             assert(i>=0)
             self.notebook.remove_page(i)
+            if self.notebook.get_n_pages() == 0:
+                self.widget.set_title("Meld")
 
     def on_file_changed(self, srcpage, filename):
         for c in self.notebook.get_children():
@@ -732,6 +758,7 @@ class MeldApp(gnomeglade.GnomeApp):
         doc = dirdiff.DirDiff(self.prefs, len(dirs))
         self._append_page(doc, "tree-folder-normal.png")
         doc.set_locations(dirs)
+        return doc
 
     def append_filediff(self, files):
         assert len(files) in (1,2,3)
@@ -742,6 +769,7 @@ class MeldApp(gnomeglade.GnomeApp):
         seq.connect("can-redo", self.on_can_redo)
         self._append_page(doc, "tree-file-normal.png")
         doc.set_files(files)
+        return doc
 
     def append_diff(self, paths):
         aredirs = [ os.path.isdir(p) for p in paths ]
@@ -751,9 +779,9 @@ class MeldApp(gnomeglade.GnomeApp):
                     parent = self,
                     buttonstype = gtk.BUTTONS_OK)
         elif 1 in aredirs:
-            self.append_dirdiff(paths)
+            return self.append_dirdiff(paths)
         else:
-            self.append_filediff(paths)
+            return self.append_filediff(paths)
 
     def append_vcview(self, locations):
         assert len(locations) in (1,)
@@ -761,6 +789,7 @@ class MeldApp(gnomeglade.GnomeApp):
         doc = vcview.VcView(self.prefs)
         self._append_page(doc, "vc-icon.png")
         doc.set_location(location)
+        return doc
 
     #
     # Current doc actions
@@ -793,22 +822,6 @@ class MeldApp(gnomeglade.GnomeApp):
 # usage
 #
 ################################################################################
-usage_string = _("""Meld is a file and directory comparison tool. Usage:
-
-    meld                        Start with no windows open
-    meld <dir>                  Start with VC browser in 'dir'
-    meld <file>                 Start with VC diff of 'file'
-    meld <file> <file> [file]   Start with 2 or 3 way file comparison
-    meld <dir>  <dir>  [dir]    Start with 2 or 3 way directory comparison
-
-Options:
-    -h, --help                  Show this help text and exit
-    -v, --version               Display the version and exit
-
-For more information choose help -> contents.
-Report bugs at http://bugzilla.gnome.org/buglist.cgi?product=meld
-Discuss meld at http://mail.gnome.org/mailman/listinfo/meld-list
-""")
 
 version_string = _("""Meld %s
 Written by Stephen Kennedy <stevek@gnome.org>""") % version
@@ -819,6 +832,7 @@ Written by Stephen Kennedy <stevek@gnome.org>""") % version
 #
 ################################################################################
 def main():
+    import optparse
     class Unbuffered(object):
         def __init__(self, file):
             self.file = file
@@ -828,17 +842,25 @@ def main():
         def __getattr__(self, attr):
             return getattr(self.file, attr)
     sys.stdout = Unbuffered(sys.stdout)
-    args = sys.argv[1:]
 
-    if args:
-        if "-h" in args or "--help" in args:
-            print usage_string
-            return
-        if "-v" in args or "--version" in args:
-            print version_string
-            return
+    parser = optparse.OptionParser(
+    usage="""
+    %prog                       Start with no windows open
+    %prog <dir>                 Start with VC browser in 'dir'
+    %prog <file>                Start with VC diff of 'file'
+    %prog <file> <file> [file]  Start with 2 or 3 way file comparison
+    %prog <dir>  <dir>  [dir]   Start with 2 or 3 way directory comparison""",
+    description="""Meld is a file and directory comparison tool.""",
+    version="%prog "+version)
+    parser.add_option("-L", "--label", action="append", default=[], help=_("Set label to use instead of file name"))
+    parser.add_option("-u", "--unified", action="store_true", help=_("Ignored for compatibility"))
+    parser.add_option("-c", "--context", action="store_true", help=_("Ignored for compatibility"))
+    parser.add_option("-e", "--ed", action="store_true", help=_("Ignored for compatibility"))
+    parser.add_option("-r", "--recursive", action="store_true", help=_("Ignored for compatibility"))
+    options, args = parser.parse_args()
 
     app = MeldApp()
+    tab = None
 
     if len(args) == 0:
         pass
@@ -855,12 +877,14 @@ def main():
             doc.connect("create-diff", lambda obj,arg: app.append_diff(arg) )
             doc.run_diff([a])
         else:
-            app.append_vcview( [a] )
+            tab = app.append_vcview( [a] )
                 
     elif len(args) in (2,3):
-        app.append_diff(args)
+        tab = app.append_diff(args)
     else:
         app.usage( _("Wrong number of arguments (Got %i)") % len(args))
 
+    if tab:
+        tab.set_labels( options.label )
     app.main()
 
